/*
 * Copyright (c) 1995-1999 Joey Hess (pdmenu@joeyh.name)
 * All rights reserved. See COPYING for full copyright information (GPL).
 */
   
void DrawAll(void);
void Force_Redraw(void);
void Resize_Screen(void);
void AddWindow(Menu_Type *);
void ExitWindow();
int IsVisible(Menu_Type *);
